function [a bb] = OptimizeDesign(a,niter)
%function [A b uu]=OptimizeDesign(a)
% OPTIMIZEDESIGN is a function that optimize the design defined in an 
% object of class fluidpath.
% The vector uu is assigned a fixed structure.
% If elements in uu are not to be optimized but fixed as given in a,
% they are fixed by upper and lower bound conditions. 
%
% uu=[uM(2) uM3' uM3 uM4' uM4                2*ns-1    1..2*ns-1
%    [dxi(1)..dxi(ns)                          ns      2*ns..3*ns-1
%    [VRoverF0                                 1       3*ns
%    [uH(1) uH(2) ...uH(nH)                    ns      3*ns+1..4*ns         
%    [uT(1) uT(2) ...uT(nT)                    ns      4*ns+1..5*ns         
%    [uA(1) uA(2) ...uA(ns)                    ns      5*ns+1..6*ns         
%    [uF(1) uF(2) ...uF(ns)                    ns      6*ns+1..7*ns 
%
% A*uu<=b 
% Aeq*uu=beq
% 
ns = a.ns;                  %% Number of stages
n = 3*ns-1;                 %% Least number of optimization variables

nV = a.update_sigma;        %% Is 0 or 1
n = n+nV;

a.puH = n;                  %% Heat exchange
a.iuH = find(a.update_uH==1);
a.luH = length(a.iuH);
n = n + a.luH; 

a.puT = n;                  %% Temperature cooling media
a.iuT = find(a.update_uT==1);
a.luT = length(a.iuT);
n = n + a.luT;

a.puA = n;                  %% Catalyst distribution
a.iuA = find(a.update_uA==1);
a.luA = length(a.iuA);
n = n + a.luA;

a.puF = n;                  %% Feed distribution
a.iuF = find(a.update_uF==1);
a.luF = length(a.iuF);
n = n + a.luF;

a.puFF = n;                 %% Second feed distribution
a.iuFF = find(a.update_uFF==1);
a.luFF = length(a.iuFF);
n = n + a.luFF;

a.puS = n;                  %% Separation
a.iuS = find(a.update_uS==1);
a.luS = length(a.iuS);
n = n + a.luS;

a.ndof = n;                 %% Degrees of freedom
n = n + (a.ncol+2)*a.ns*a.nz;

 %n =n+a.nxx; %mhi 31Augu07
 %a.key=n;
 
% n is total number of variables to be optimized
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set linear inequality constraints on uM
% A*uM<=b
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%A = zeros(2*ns-1,n);b=zeros(2*ns-1,1);
A = zeros(3*ns-2,n); b = zeros(3*ns-2,1);
for i=1:2*ns-1, A(i,i)  = 1.0; end
for i=1:2*ns-2, A(i+1,i)=-1.0; end
%for i=1:ns, b(2*i-1)=a.dxi(i); end
 %for i=1:ns, A(2*i-1,2*ns+i)=-1.0; end
for i=1:ns, A(2*i-1,2*ns+i-1)=-1.0; end %@
for i=1:ns-1, A(2*ns-1+i,2*i)=1; end
for i=1:ns-1, A(2*ns-1+i,2*i+1)=-1; end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set linear equality constraints on dxi sum(dxi(i))=1
% and 1.0*u=a.u where u is constant
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Aeq = zeros(1,n);beq=zeros(1,1);
%Aeq(1,2*ns+1:3*ns)=1.0; beq(1)=1.0;
Aeq(1,2*ns:3*ns-1)=1.0; beq(1)=1.0; %@

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set lower and upper bounds on uu
% uu=[uM1 uM1' uM2 uM2' uM3                2*ns-1      1..2*ns-1
%    [dxi(1)..dxi(ns)                          ns      2*ns..3*ns-1
%    [VRoverF0                                 1       3*ns
%    [uH(1) uH(2) ...uH(ns)                    ns      3*ns+1..4*ns         
%    [uT(1) uT(2) ...uT(ns)                    ns      4*ns+1..5*ns         
%    [uA(1) uA(2) ...uA(ns)                    ns      5*ns+1..6*ns         
%    [uF(1) uF(2) ...uF(ns)                    ns      6*ns+1..7*ns 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lb=zeros(n,1); 
ub=zeros(n,1);
lb(1:n)  = 0.0;                 % No design function can be negative
ub(1:n)  = inf;                 % Need to be modified
lb(2*ns:3*ns-1)=1/a.ns*0.1;     % Minimum dxi 
ub(2*ns:3*ns-1)=1.0;            % dxi is max 1.0
if nV==1, 
    lb(3*ns) = a.sigma_min;     % Minimum VRoverF0
    ub(3*ns) = a.sigma_max;     % Maximum VRoverF0
end
if a.luH>0,
    lb(a.puH+1:a.puH+a.luH)=a.uH_min(a.iuH);    % Min heat transfer area
    ub(a.puH+1:a.puH+a.luH)=a.uH_max(a.iuH);    % Max heat transfer area	
end
if a.luT>0,
    lb(a.puT+1:a.puT+a.luT)=a.uT_min(a.iuT);    % Min coolant temperature
    ub(a.puT+1:a.puT+a.luT)=a.uT_max(a.iuT);    % Max coolant temperature
end
if a.luA>0,
    lb(a.puA+1:a.puA+a.luA)=a.uA_min(a.iuA);    % Min catalyst activity
    ub(a.puA+1:a.puA+a.luA)=a.uA_max(a.iuA);    % Max catalyst activity
end
if a.luF>0,
    lb(a.puF+1:a.puF+a.luF)=a.uF_min(a.iuF);    % Min extra feed 
    ub(a.puF+1:a.puF+a.luF)=a.uF_max(a.iuF);    % Max extra feed
end

if a.luFF>0,
    lb(a.puFF+1:a.puFF+a.luFF)=a.uFF_min(a.iuFF);  % Min extra feed 
    ub(a.puFF+1:a.puFF+a.luFF)=a.uFF_max(a.iuFF);  % Max extra feed 
end

if a.luS>0,
    lb(a.puS+1:a.puS+a.luS)=a.uS_min(a.iuS);       % Min extra feed
    ub(a.puS+1:a.puS+a.luS)=a.uS_max(a.iuS);       % Max extra feed
end
% No constraints on Z.
lb(a.ndof+1:n)=-inf;
ub(a.ndof+1:n)= inf;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Optimize the design
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
uu=AssignVector(a);
%uu(1:a.ndof)=uu(1:a.ndof)+rand((a.ndof),1);
uu0=uu;
%bb=uu;
%    'HessUpdate','bfgs',...
%    'LevenbergMarquardt','on',...
%return 
    %'TolFun',1.e-5,'TolX',1.e-5,'Display','iter','MaxIter',niter); 
options = optimset('Algorithm','active-set','TolCon',a.tol(1),...
    'TolFun',a.tol(2),'Display','iter','HessUpdate','bfgs',...
    'MaxFunEval',100000000,'MaxIter',niter,'UseParallel','always'); 
[uu,fval,exitflag,output,lambda,grad,hessian] = ...
            fmincon(@(uu) ObjectiveFunction(uu,a),uu0,A,b,Aeq,beq,lb,ub,...
                    @(uu) ModelConstraints(uu,a),options);
display(exitflag)
disp(A*uu-b);
disp(Aeq*uu-beq);

a=AssignObject(a,uu);
bb.uu=uu;
bb.fval=fval;
bb.exitflag=exitflag;
bb.output=output;
bb.lambda=lambda;
bb.grad=grad;
bb.hessian=hessian;
return