%% Fischer-Tropsch, reactor staging
clear all
close all
clc

%% Local variables 
c = str2mat('CO','CO2','H2','H2O','CH4','C2H6','C3-4','C5-10','C11+');

Tf = 200;                %% Temperature feed [C]
Tw = 205;                %% Temperature cooling [C]
Tref = 200 + 273;        %% Reference temperature
pTot = 2e6;              %% Pressure [Pa]
bulkCatDensity = 200e03; %% Catalyst density [g/m^3]

% [M, cpnom, liqFrac] = uniSimPropertiesTodicLump(TF,pTot);
[M, cpnom, liqFrac] = flowPropTPlump(Tf,pTot); 
M = round(M);
%            1    2     3    4     5    6    7     8      9     10
s = strvcat('CO','CO2','H2','H2O','C1','C2','C34','C510','C11','T');
for i=1:size(s,1) 
    eval([s(i,:),'=int8(',num2str(i),');']);
end
%%
ns = 1;             %% Number of stages
nc = length(M);     %% Number of components
ncol = 20;          %% Number of internal collocation points

% Mole fractions fresh feed
yF0 = zeros(nc,1); 
yF0(CO2) = 0.10;
yF0(CO)  = 0.29;
yF0(H2)  = 0.61;

Mav = yF0'*M;

% Mass fractions fresh feed
yF0m = moletomass(yF0,M);

%  Mole fractions extra feed
yF = zeros(nc,1);  
yF(H2) = 1;

% Mass fractions extra feed
yFm = moletomass(yF,M);

%%
% VR = 400;         %% Reactor volume [m3]
% F0 = 5000;        %% Feed flow [mol/s]
% W0 = F0/1000*Mav; %% Feed flow [kg/s]
% sigma = VR/W0;    %% Space time [m3s/kg]
sigma = 1.4556;
uHi = 20;           %% Heat transfer area, design function
   

%% Create a object a of class fluidpath
a = fluidpath(ns,nc,ncol);
%%%%%%%%%%%%%%%%%%%%%%%%%%
u1 = ones(1,2*ns); 
a = pathset(a,'T0',Tf);                 %% Feed temperature
a = pathset(a,'Tcool0',Tw);             %% Inital cooling water temperature
a = pathset(a,'Tref',Tref);             %% Reference temperature
a = pathset(a,'pTot',pTot);             %% Total pressure
a = pathset(a,'M',M');                  %% Molar mass [kg/kmol]                 
%                    TolCon TolFun TolX
a = pathset(a,'tol',[1e-8   1e-8   1e-6  1e-6 1e-6]);	%% Tolerances
a = pathset(a,'cp',cpnom');                   %% Heat capacities [kJ/(kg*K)]
a = pathset(a,'liqFrac',liqFrac');
a = pathset(a,'kinetics','kinTodicLump');     %% Kinetic function          
a = pathset(a,'xx0',[yF0m;(Tf + 273 - Tref)/Tref],1); %% State vector
a = pathset(a,'xxF',[yFm;(Tf + 273 - Tref)/Tref],1);  %% State vector
a = pathset(a,'xxFF',[yF0m;(Tf + 273 - Tref)/Tref],1);%% State vector
a = pathset(a,'xS',[0 0 0 1 0 0 0 0.7 1]',1);  %% Separation function
a = pathset(a,'key',C11);                      %% Key component
a = pathset(a,'sigma0',sigma);                 %% Inital sigma

%%
% Parameterization of design functions
a = pathset(a,'update_sigma',0);                    %% Space time
a = pathset(a,'update_uH', repmat([1 -1], 1, ns));  %% Heat exchanger area
a = pathset(a,'update_uT', repmat([1 -1], 1, ns));  %% Temp cooling media
a = pathset(a,'update_uF', repmat([0 0], 1, ns));   %% Extra feed
a = pathset(a,'update_uFF',repmat([0 0], 1, ns));   %% 2nd extra feed
a = pathset(a,'update_uA', repmat([0 0],1,ns));     %% Catalyst activity
a = pathset(a,'update_uS', repmat([0 0],1,ns));     %% Separation
a = pathset(a,'uFp',       repmat(1,1,ns)); %% If 1: point feed 1st extra feed
a = pathset(a,'uFFp',      repmat(1,1,ns)); %% If 1: point feed 2nd extra feed
a = pathset(a,'uSp',       repmat(0,1,ns));

%%
% Initial values and boundaries of design functions
% Space time
a = pathset(a,'sigma',sigma);               
a = pathset(a,'sigma_max',5);
a = pathset(a,'sigma_min',0);

% Heat transfer area distribution
a = pathset(a,'uH',    u1*uHi/40);
a = pathset(a,'uH_min',u1*5/40);
a = pathset(a,'uH_max',u1*30/40);

% Temperature cooling media
a = pathset(a,'uT',    u1*(Tw+273-Tref)/Tref);  
a = pathset(a,'uT_min',u1*(Tw-10+273-Tref)/Tref);
a = pathset(a,'uT_max',u1*(Tw+10+273-Tref)/Tref);

% Catalyst activity
a = pathset(a,'uA',    u1);                   
a = pathset(a,'uA_min',u1*0.1);
a = pathset(a,'uA_max',u1*1);

% 1st extra feed
a = pathset(a,'uF',    u1*0);
a = pathset(a,'uF_min',u1*0);
a = pathset(a,'uF_max',u1*1);

% 2nd extra feed
a = pathset(a,'uFF',u1*0);
a = pathset(a,'uFF_min',u1*0);
a = pathset(a,'uFF_max',u1*10);

% Separation
a = pathset(a,'uS',    u1*0);
a = pathset(a,'uS_min',u1*0);
a = pathset(a,'uS_max',u1*1);

% Objective function
a = pathset(a,'Jcrit',1); 

%% Integration
tic
a = integrate(a);
toc
% CO conversion
y1 = masstomole(a.Z(end,1:nc)',M);
X_CO1 = (yF0(CO) - y1(CO))/yF0(CO);            %% Molar basis 
MX_CO1 = 1 - a.Z(end,1)/a.Z(1,1)*a.Z(end,end); %% Weight basis

%% Optimization
[a, b] = OptimizeDesign(a,5);

% CO conversion
y = masstomole(a.Z(end,1:nc)',M);
X_CO = (yF0(CO) - y(CO))/yF0(CO);             %% Molar basis
MX_CO = 1 - a.Z(end,1)/a.Z(1,1)*a.Z(end,end); %% Weight basis
fprintf('CO conversion before optimization = %0.2f\n',MX_CO1)
fprintf('CO conversion after optimization = %.2f\n',MX_CO)
plotFT(a);

save 1stage_uM_uH_uT.mat    %% Save results