function [R, alpha, varargout] = kinTodicLump(xx,uA,Cp,M,Tref,pTot)
%-------------------------------------------------------------------%
% Name  		    : kinTodicLump.m
% Author		    : Martin S. Foss
% Date              : Spring 2013
% Input:
%   xx(1)     		: Mass fraction of CO
%   xx(2)           : Mass fraction of CO2
%   xx(3)     		: Mass fraction of H2
%   xx(4)     		: Mass fraction of H2O
%   xx(5)     		: Mass fraction of C1 
%   xx(6)     		: Mass fraction of C2 
%   xx(7)           : Mass fraction of C3-4 
%   xx(8)           : Mass fraction of C5-10
%   xx(9)           : Mass fraction of C11+
%   xx(10)     		: Temperature (T-Tref)/Tref [-], T = [K]
%   uA              : Relative activity (catalyst dilution)
%   cp              : Average cp of composition [kJ/(kg*K)]
%   M(1:6)          : Molecular weight [kg/kmol]
%   Tref            : Reference temperature [K]
%   pTot            : Total pressuse [Pa]
% Output:
%   R(i)            : Component reaction rates [kg/(m3*s)]
%   alpha           : Growth probobility
%
%   Reactions taking place:
%   iCO + (2i + 1)H2 = Ci + iH2O, i = 1..15	(Paraffins)
%       iCO + (2i)H2 = Ci + iH2O, i = 2..15 (Olefins) 
%-------------------------------------------------------------------%

Rgas = 8.314472e-03;        %% Gas constant [kJ/K*mol] 
p = pTot/1e06;              %% Pressure [MPa]
T = Tref*(xx(end) + 1);     %% Temperature [K]

%% Parameters
A1  = 1.83e10;      %% [mol/(gcat*h*MPa)]
A2  = 5.08e00;      %% [-]
A3  = 2.44e01;      %% [MPa^-1]
A4  = 2.90e00;      %% [-]
A5  = 4.49e05;      %% [mol/(gcat*h*MPa)]
A6  = 7.47e08;      %% [mol/(gcat*h)]
A7  = 1.00e-03;     %% [MPa^-1]
A5M = 8.43e05;      %% [mol/(gcat*h*MPa)]
A6E = 7.03e08;      %% [mol/(gcat*h)]
E1  = 100.4;        %% [kJ/mol]
E5  = 72.4;         %% [kJ/mol]
E5M = 63.0;         %% [kJ/mol]
E60 = 97.2;         %% [kJ/mol]
E6E0 = 108.8;       %% [kJ/mol]
deltaH2 = 8.68;     %% [kJ/mol] 
deltaH3 = 9.44;     %% [kJ/mol] 
deltaH4 = 7.9;      %% [kJ/mol]
deltaH7 = -25.0;    %% [kJ/mol]
deltaE  = 1.12;     %% [kJ/(mol_CH2)]

%% Kinetic constants
k1  = A1*exp(-E1/(Rgas*T));        %% [mol/(gcat*h*MPa)]
K2  = A2*exp(-deltaH2/(Rgas*T));   %% [-]
K3  = A3*exp(-deltaH3/(Rgas*T));   %% [MPa^-1]
K4  = A4*exp(-deltaH4/(Rgas*T));   %% [-]
k5M = A5M*exp(-E5M/(Rgas*T));      %% [mol/(gcat*h*MPa)]
k5  = A5*exp(-E5/(Rgas*T));        %% [mol/(gcat*h*MPa)]
k6E = A6E*exp(-E6E0/(Rgas*T));     %% [mol/(gcat*h)]
k60 = A6*exp(-E60/(Rgas*T));       %% [mol/(gcat*h)]  
K7  = A7*exp(-deltaH7/(Rgas*T));   %% [Mpa^-1]

%%
%Assume sharp split: all components with carbon number greater than 5 in liquid phase
gasPhase = xx(1:7);
molarGas = M(1:7);

omegaGas = gasPhase./sum(gasPhase);
moleFracGas = masstomole(omegaGas,molarGas);

partialP = moleFracGas*p;       %% [MPa]
pCO = partialP(1);              %% [MPa]
pH2 = partialP(3);              %% [MPa]
pH2O = partialP(4);             %% [MPa]

c = -deltaE/(Rgas*T);

alpha = zeros(15,1);
alpha(1) = k1*pCO/(k1*pCO + k5M*pH2);
alpha(2) = k1*pCO/(k1*pCO + k5*pH2 + k6E*exp(2*c));

for i=3:15
    alpha(i) = k1*pCO/(k1*pCO + k5*pH2 + k60*exp(i*c));
end

alphaSum = 0;
for i=3:15
    alphaSum = alphaSum + prod(alpha(3:i));
end

S = 1/(1 + sqrt(K7*pH2) + sqrt(K7*pH2)*(1 + 1/K4 + 1/(K3*K4*pH2) ...
    + pH2O/(K2*K3*K4*pH2^2))*(alpha(1) + prod(alpha(1:2)) + ...
    prod(alpha(1:2))*alphaSum));     %% Fraction of vacant sites

rCH4  = k5M*K7^0.5*pH2^1.5*alpha(1)*S;                  %% [mol/(gcat*h)]
rC2H4 = k6E*exp(2*c)*sqrt(K7*pH2)*prod(alpha(1:2))*S;   %% [mol/(gcat*h)]

rParaffin    = zeros(15,1);
rParaffin(1) = rCH4;            %% [mol/(gcat*h)]

rOlefin    = zeros(15,1);
rOlefin(2) = rC2H4;             %% [mol/(gcat*h)]

for i=2:15
    rParaffin(i) = k5*K7^0.5*pH2^1.5*prod(alpha(1:2))*prod(alpha(3:i));
end
for i=3:15
    rOlefin(i)   = k60*exp(c*i)*sqrt(K7*pH2)*prod(alpha(1:2))* ...
                   prod(alpha(3:i));
end

bulkCatDensity = 200e03;   %% [g/m^3]
kmole = 1000;              %% [mol/kmol]
sperh = 3600;              %% [s/h]

rParaffin = rParaffin*bulkCatDensity*kmole^(-1)*sperh^(-1); %% [kmol/(m^3*s)]
rOlefin = rOlefin*bulkCatDensity*kmole^(-1)*sperh^(-1);     %% [kmol/(m^3*s)]

rCO = 0;
rH2 = 0;
for i = 1:15
    rCO = rCO + i*(rParaffin(i) + rOlefin(i));
    rH2 = rH2 + ((2*i + 1)*rParaffin(i) + 2*i*rOlefin(i));
end

%% Lumping components:
rC1   = rParaffin(1);
rC2   = rParaffin(2) + rOlefin(2);
rC34  = sum(rParaffin(3:4)) + sum(rOlefin(3:4));
rC510 = sum(rParaffin(5:10)) + sum(rOlefin(5:10));
rC11  = sum(rParaffin(11:end)) + sum(rOlefin(11:end));

Mwhole = intMolarMass;      %% Molecular weight, whole numbers (C1 - C15)
Mpar = Mwhole(5:19);        %% Molecular weight paraffins
Mole = [0; Mwhole(20:end)]; %% Molecular weight olefins
M2   = (rParaffin(2)*Mpar(2) + rOlefin(2)*Mole(2))/ ...
       (rC2);
M34  = (rParaffin(3:4)'*Mpar(3:4) + rOlefin(3:4)'*Mole(3:4))/(rC34);
M510 = (rParaffin(5:10)'*Mpar(5:10) + rOlefin(5:10)'*Mole(5:10))/(rC510);
M11  = (rParaffin(11:end)'*Mpar(11:end) + rOlefin(11:end)'*Mole(11:end))/(rC11);

Mtemp = [M(1:5);
         M2;
         M34;
         M510;
         M11];

r = [-rCO;     %% CO
     0;        %% CO2
     -rH2;     %% H2
     rCO;      %% H2O
     rC1       %% C1
     rC2;      %% C2
     rC34;     %% C3-4
     rC510;    %% C5-10
     rC11];    %% C11+ [kmol/(m^3*s)]

g = uA*r.*Mtemp;               %% [kg/(m^3*s)]
massBalance = sum(g);
if massBalance > 0.0000001
     error('error in mass balance')
end
rTemp = [-rCO;              %% CO       [kmol/(m^3*s)]
          0;                %% CO2
         -rH2;              %% H2
          rCO;              %% H2O
          rParaffin;        %% CnH(2n+2)
          rOlefin(2:15)];   %% CnH(2n)

deltaH1 = deltaHrx(rTemp);  %% [kJ/(m^3*s)]
hh = -deltaH1/(Cp*Tref);    %% [kg/(m^3*s)]
R = [g; hh];                %% [kg/(m^3*s)]
return
